/*
 * Ambient - the ultimate desktop
 * ------------------------------
 * Copyright 2001-2005 by David Gerber <zapek@morphos.net>
 * Copyright 2006 Ambient Open Source Team
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Id: panelitem.c,v 1.9 2007/07/08 20:56:42 fab Exp $
 */

#include "ambient.h"

/* public */

/* private */
#include "ambient_cat.h"
#include "panelitem.h"
#include "gfx_bitmap.h"
#include "gfx_blit.h"
#include "panelitem_button_logo.h"
#include "mui_func.h"


#ifndef DEPEND
#if PANELIMAGE_WIDTH != PANELITEM_BUTTON_WIDTH
#error "panelitem_button width doesn't match"
#endif

#if PANELIMAGE_HEIGHT != PANELITEM_BUTTON_HEIGHT
#error "panelitem_button height doesn't match"
#endif
#endif

struct itemlist_entry *create_panelitem(ULONG type, CONST_STRPTR path)
{
	struct itemlist_entry *ie = NULL;

	switch (type)
	{
		case PT_BUTTON:
			{
				if ( (ie = malloc(sizeof(*ie))) )
				{
					ie->type     = type;
					ie->name     = GSI(MSG_PANELITEM_BUTTON);
					ie->author   = "David Gerber, Ambient Open Source Team";
					ie->descr    = GSI(MSG_PANELITEM_BUTTONDESC);
					ie->version  = 1;
					ie->revision = 1;
					ie->image    = NULL; /* XXX: important.. do it for everything! */
					ie->cl       = getpanelbuttonclass();

					if ( (ie->bm = gfx_bitmap_create(PANELIMAGE_WIDTH, PANELIMAGE_HEIGHT, 32, BITMAPTAG_Format, BITMAPVAL_Format_ARGB32, TAG_DONE)) )
					{
						gfx_blit(panelitem_button, ie->bm,
							BLITTAG_SrcType, BLITVAL_SrcType_Array,
							BLITTAG_SrcFormat, BLITVAL_SrcFormat_ARGB,
						TAG_DONE);
					}
					/* If it fails, no image then */
				}
				/* XXX */
			}
			break;

		case PT_MENU:

			break;

		case PT_ZAPLET:

			break;

		case PT_SPACER:
			{
				if ( (ie = malloc(sizeof(*ie))) )
				{
					ie->type     = type;
					ie->name     = GSI(MSG_PANELITEM_SPACER);
					ie->author   = "Vladimir Alaev, Ambient Open Source Team";
					ie->descr    = GSI(MSG_PANELITEM_SPACERDESC);
					ie->version  = 1;
					ie->revision = 1;
					ie->image    = NULL; /* XXX: important.. do it for everything! */
					ie->cl       = getpanelspacerclass();

					if ( (ie->bm = gfx_bitmap_create(PANELIMAGE_WIDTH, PANELIMAGE_HEIGHT, 32, BITMAPTAG_Format, BITMAPVAL_Format_ARGB32, TAG_DONE)) )
					{
						gfx_blit(panelitem_button, ie->bm,
							BLITTAG_SrcType, BLITVAL_SrcType_Array,
							BLITTAG_SrcFormat, BLITVAL_SrcFormat_ARGB,
						TAG_DONE);
					}
					/* If it fails, no image then */
				}
				/* XXX */
			}
			break;

		case PT_SEPARATOR:
			{
				if ( (ie = malloc(sizeof(*ie))) )
				{
					ie->type     = type;
					ie->name     = GSI(MSG_PANELITEM_SEPARATOR);
					ie->author   = "Christian Rosentreter, Ambient Open Source Team";
					ie->descr    = GSI(MSG_PANELITEM_SEPARATORDESC);
					ie->version  = 1;
					ie->revision = 0;
					ie->image    = NULL; /* XXX: important.. do it for everything! */
					ie->cl       = getpanelseparatorclass();

					if ( (ie->bm = gfx_bitmap_create(PANELIMAGE_WIDTH, PANELIMAGE_HEIGHT, 32, BITMAPTAG_Format, BITMAPVAL_Format_ARGB32, TAG_DONE)) )
					{
						gfx_blit(panelitem_button, ie->bm,
							BLITTAG_SrcType,   BLITVAL_SrcType_Array,
							BLITTAG_SrcFormat, BLITVAL_SrcFormat_ARGB,
						TAG_DONE);
					}
					/* If it fails, no image then */
				}
				/* XXX */
			}
			break;

		case PT_VIEWWATCHER:
			{
				if ( (ie = malloc(sizeof(*ie))) )
				{
					ie->type     = type;
					ie->name     = GSI(MSG_PANELITEM_VIEWWATCHER);
					ie->author   = "Vladimir Alaev, Ambient Open Source Team";
					ie->descr    = GSI(MSG_PANELITEM_VIEWWATCHERDESC);
					ie->version  = 1;
					ie->revision = 0;
					ie->image    = NULL; /* XXX: important.. do it for everything! */
					ie->cl       = getpanelviewwatcherclass();

					if ( (ie->bm = gfx_bitmap_create(PANELIMAGE_WIDTH, PANELIMAGE_HEIGHT, 32, BITMAPTAG_Format, BITMAPVAL_Format_ARGB32, TAG_DONE)) )
					{
						gfx_blit(panelitem_button, ie->bm,
							BLITTAG_SrcType,   BLITVAL_SrcType_Array,
							BLITTAG_SrcFormat, BLITVAL_SrcFormat_ARGB,
						TAG_DONE);
					}
					/* If it fails, no image then */
				}
				/* XXX */
			}
			break;

		case PT_BOOKMARKS:
			{
				if ( (ie = malloc(sizeof(*ie))) )
				{
					ie->type     = type;
					ie->name     = GSI(MSG_PANELITEM_BOOKMARKS);
					ie->author   = "Michal Wozniak, Ambient Open Source Team";
					ie->descr    = GSI(MSG_PANELITEM_BOOKMARKSDESC);
					ie->version  = 1;
					ie->revision = 0;
					ie->image    = NULL; /* XXX: important.. do it for everything! */
					ie->cl       = getpanelbookmarksclass();

					if ( (ie->bm = gfx_bitmap_create(PANELIMAGE_WIDTH, PANELIMAGE_HEIGHT, 32, BITMAPTAG_Format, BITMAPVAL_Format_ARGB32, TAG_DONE)) )
					{
						gfx_blit(panelitem_button, ie->bm,
							BLITTAG_SrcType,   BLITVAL_SrcType_Array,
							BLITTAG_SrcFormat, BLITVAL_SrcFormat_ARGB,
						TAG_DONE);
					}
					/* If it fails, no image then */
				}
				/* XXX */
			}
			break;


#if USE_TRAY_LIB
		case PT_TRAY_PANEL:
			{
				if ( (ie = malloc(sizeof(*ie))) )
				{
					ie->type     = type;
					ie->name     = GSI(MSG_PANELITEM_TRAYPANEL);
					ie->author   = "Vladimir Alaev";
					ie->descr    = GSI(MSG_PANELITEM_TRAYPANELDESC);
					ie->version  = 0;
					ie->revision = 1;
					ie->image    = NULL; /* XXX: important.. do it for everything! */
					ie->cl       = gettraypanelclass();

					if ( (ie->bm = gfx_bitmap_create(PANELIMAGE_WIDTH, PANELIMAGE_HEIGHT, 32, BITMAPTAG_Format, BITMAPVAL_Format_ARGB32, TAG_DONE)) )
					{
						gfx_blit(panelitem_button, ie->bm,
							BLITTAG_SrcType, BLITVAL_SrcType_Array,
							BLITTAG_SrcFormat, BLITVAL_SrcFormat_ARGB,
						TAG_DONE);
					}
					/* If it fails, no image then */
				}
				/* XXX */
			}
#endif
			break;

		#ifdef DEBUG
		default:
			PDB(("out of bounds\n"));
			break;
		#endif
	}
	return (ie);
}


void delete_panelitem(struct itemlist_entry *ie)
{
	ASSERT(ie);

	gfx_bitmap_delete(ie->bm);

	switch (ie->type)
	{
		case PT_BUTTON:
			free(ie);
			break;

		case PT_MENU:

			break;

		case PT_ZAPLET:

			break;

		case PT_SPACER:
			free(ie);
			break;

		case PT_VIEWWATCHER:
			free(ie);
			break;         

		case PT_BOOKMARKS:
			free(ie);
			break;

#ifdef USE_TRAY_LIB
		case PT_TRAY_PANEL:
			free(ie);
			break;
#endif
		#ifdef DEBUG
		default:
			PDB(("out of bounds\n"));
			break;
		#endif
	}
}

LONG panelitem_name_to_type(CONST_STRPTR name)
{
	if(!stricmp("button", name))
		return MV_Panel_Type_Button;
	else if(!stricmp("separator", name))
		return MV_Panel_Type_Separator;
	else if(!stricmp("spacer", name))
		return MV_Panel_Type_Spacer;
	else if(!stricmp("viewwatcher", name))
		return MV_Panel_Type_ViewWatcher;
	else if(!stricmp("bookmarks", name))
		return MV_Panel_Type_Bookmarks;

#if USE_TRAY_LIB
	else if(!stricmp("tray", name))
		return MV_Panel_Type_TrayPanel;
#endif
	else /* XXX: Oh rly? */
		return MV_Panel_Type_Button;
}

