/*
 * Ambient - the ultimate desktop
 * ------------------------------
 * Copyright 2001-2005 by David Gerber <zapek@morphos.net>
 * Copyright 2005-2006 Ambient Open Source Team
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Id: panelpopupclass.c,v 1.1 2007/07/08 20:56:42 fab Exp $
 */

#include "ambient.h"

/* public */
#include <cybergraphx/cybergraphics.h>
#include <graphics/rpattr.h>
#include <proto/cybergraphics.h>
#include <proto/graphics.h>

/* private */
#include "mui_func.h"
#include "gfx_bitmap.h"
#include "gfx_blit.h"
#include "gfx_scale.h"
#include "deficonpool.h"
#include "rexx.h"
#include "command.h"
#include "contextmenu.h"
#include "name.h"
#include "prefs.h"
#include "file_func.h"
#include "methodstack.h"
#include "legacy.h"
#include "prefs_advanced.h"
#include "viewwatcher_arrow_logo.h"

struct Data {
	ULONG  distance;
	APTR   cmenu;
	ULONG  views;
	APTR   bm;
	APTR   selbm;
	APTR   scaled;
	APTR   selscaled;
	ULONG  oldwidth;
	ULONG  oldheight;
	ULONG  pressed;
	LONG   icontype;
	STRPTR menutitle;
};

static void load_icon(APTR obj, struct Data *data, APTR *bm, APTR *selbm)
{
	APTR o = NULL;
	APTR bm2 = NULL, bm3 = NULL;

	if ( (o = (APTR)methodstack_push_sync(app, 3, MM_Application_CreateIcon, TRUE, MV_ViewID_Panel)))
	{
		if (deficonpool_apply_default_icon(o, data->icontype, NULL, NULL))
		{
			APTR bm1;

			bm1 = (APTR)getv(o, MA_Icon_ImageNormal);
			if (bm1 != NULL)
			{
				bm2 = gfx_bitmap_create(gfx_bitmap_width(bm1), gfx_bitmap_height(bm1), gfx_bitmap_depth(bm1),
											BITMAPTAG_Clear, TRUE, BITMAPTAG_Friend, bm1, TAG_DONE);
				
				bm3 = gfx_bitmap_create(gfx_bitmap_width(bm1), gfx_bitmap_height(bm1), gfx_bitmap_depth(bm1),
											BITMAPTAG_Clear, TRUE, BITMAPTAG_Friend, bm1, TAG_DONE);

				if (bm2 != NULL && bm3 != NULL)
				{
					APTR scaled_arrow, arrow;
					struct RastPort rp;

                    gfx_blit(bm1, bm2, TAG_DONE);
					gfx_blit(bm1, bm3, TAG_DONE);

					arrow = gfx_bitmap_create(VIEWWATCHER_ARROW_WIDTH, VIEWWATCHER_ARROW_HEIGHT, 32,
											BITMAPTAG_Clear, TRUE, BITMAPTAG_Friend, bm1, TAG_DONE);

					gfx_blit(viewwatcher_arrow, arrow, BLITTAG_SrcType, BLITVAL_SrcType_Array,
													 BLITTAG_SrcFormat, BLITVAL_SrcFormat_ARGB,
													 BLITTAG_DstWidth, VIEWWATCHER_ARROW_WIDTH,
													 BLITTAG_DstHeight, VIEWWATCHER_ARROW_HEIGHT,
													 BLITTAG_Alpha, 0xffffffff,
													 TAG_DONE);

					scaled_arrow = gfx_bitmap_create((ULONG)gfx_bitmap_width(bm1)/3, (ULONG)gfx_bitmap_height(bm1)/3, gfx_bitmap_depth(bm1),
											BITMAPTAG_Clear, TRUE, BITMAPTAG_Friend, bm1, TAG_DONE);

					gfx_scale(arrow, scaled_arrow, gfx_bitmap_width(scaled_arrow), gfx_bitmap_height(scaled_arrow),
						SCALETAG_Scale2x, TRUE,
						SCALETAG_Bilinear, TRUE,
						SCALETAG_Average, TRUE,
						SCALETAG_AspectX, NULL,
						SCALETAG_AspectY, NULL,
						TAG_DONE);
					

					gfx_blit(scaled_arrow, bm2,  BLITTAG_DstX, gfx_bitmap_width(bm2) - gfx_bitmap_width(scaled_arrow),
												 BLITTAG_Modulo, gfx_bitmap_width(scaled_arrow) * 32 / 8,
												 BLITTAG_DstWidth, gfx_bitmap_width(scaled_arrow),
												 BLITTAG_DstHeight, gfx_bitmap_height(scaled_arrow),
												 BLITTAG_Alpha, 0xffffffff,
												 TAG_DONE);

					InitRastPort(&rp);
					rp.BitMap = gfx_bitmap_bm(scaled_arrow);
					ProcessPixelArray(&rp, 0, 0, gfx_bitmap_width(scaled_arrow), gfx_bitmap_height(scaled_arrow), POP_BRIGHTEN, 0x20, NULL);

					gfx_blit(scaled_arrow, bm3,  BLITTAG_DstX, gfx_bitmap_width(bm3) - gfx_bitmap_width(scaled_arrow),
												 BLITTAG_Modulo, gfx_bitmap_width(scaled_arrow) * 32 / 8,
												 BLITTAG_DstWidth, gfx_bitmap_width(scaled_arrow),
												 BLITTAG_DstHeight, gfx_bitmap_height(scaled_arrow),
												 BLITTAG_Alpha, 0xffffffff,
												 TAG_DONE);

					gfx_bitmap_delete(arrow);
					gfx_bitmap_delete(scaled_arrow);
				}
			}
        }
		methodstack_push(o, 1, OM_DISPOSE);
	}

	*bm = bm2;
	*selbm = bm3;
}

static void doset(APTR obj, struct Data *data, struct TagItem *tags)
{
	ULONG update = 0;

	FORTAG(tags)
	{
		case MA_Panel_Distance:
			data->distance = tag->ti_Data;
			break;

		case MA_Panelgroup_Horiz:
			break;

		case MUIA_Selected:
			data->pressed = tag->ti_Data;
			MUI_Redraw(obj, MADF_DRAWOBJECT);
            break;
	}
	NEXTTAG 
}


DEFNEW
{
	obj = DoSuperNew(cl, obj,
		InnerSpacing(0,0),
		MUIA_ContextMenu, 1,
		MUIA_InputMode, MUIV_InputMode_RelVerify,
		MUIA_ShowSelState, FALSE,
		MUIA_DoubleBuffer, TRUE,
		TAG_DONE
	);

	if (obj != NULL)
	{
		GETDATA;
	
		data->views = 0;
		data->scaled = NULL;
		data->selscaled = NULL;
		data->oldwidth = 0;
		data->oldheight = 0;
		data->pressed = 0;
		data->icontype = GetTagData(MA_PanelPopup_IconType, MV_Icon_Type_View, INITTAGS);
		data->menutitle = (STRPTR)GetTagData(MA_PanelPopup_MenuTitle, NULL, INITTAGS);

		load_icon(obj, data, &data->bm, &data->selbm);

		doset(obj, data, INITTAGS);	

		DoMethod(obj, MUIM_Notify, MUIA_Pressed, FALSE, obj, 1, MM_PanelPopup_Popup);
	}  

	return ((ULONG)obj);
}

DEFDISP
{
	GETDATA;

	if (data->scaled)
		gfx_bitmap_delete(data->scaled);

	if (data->selscaled)
		gfx_bitmap_delete(data->selscaled);

	if (data->bm)
		gfx_bitmap_delete(data->bm);
	
	if (data->selbm)
		gfx_bitmap_delete(data->selbm);

	return DOSUPER;
}

DEFMMETHOD(AskMinMax)
{
	ULONG size = getv(_parent(obj), MA_Panelgroup_Size);

	DOSUPER;

	ASSERT(size);

	msg->MinMaxInfo->MinWidth = size;
	msg->MinMaxInfo->MinHeight = size;
	msg->MinMaxInfo->MaxWidth = size;
	msg->MinMaxInfo->MaxHeight = size;

	return (0);
}


DEFGET
{
	GETDATA;

	switch (msg->opg_AttrID)
	{
		case MA_Panel_Distance:
			*msg->opg_Storage = data->distance;
			return (TRUE);
			break;

		case MA_Panel_Properties:
            *msg->opg_Storage = TRUE;
			return (TRUE);
			break;
	}

	return DOSUPER;
}


DEFSET
{
	GETDATA;

	doset(obj, data, INITTAGS);

	return (DOSUPER);
}


DEFMMETHOD(Draw)
{
	GETDATA;

	ULONG rc = DOSUPER;

	if (msg->flags & (MADF_DRAWOBJECT|MADF_DRAWUPDATE))
	{
		ULONG mleft, mtop, mwidth, mheight;
		ULONG txs;
		ULONG tys;
		ULONG updated = FALSE;
		APTR  tbm, scaled = data->scaled, selscaled = data->selscaled;

        mleft = _mleft(obj);
		mtop = _mtop(obj);
		mwidth = _mwidth(obj);
		mheight = _mheight(obj);


		if ((data->oldwidth != mwidth) || (data->oldheight != mheight))
		{
			if (scaled)
				gfx_bitmap_delete(scaled);

			scaled = NULL;

			if (selscaled)
				gfx_bitmap_delete(selscaled);

			selscaled = NULL;

			if ((gfx_bitmap_width(data->bm) > mwidth) || (gfx_bitmap_height(data->bm) > mheight))
			{
				gfx_scale_calc_aspect_constraints(
					gfx_bitmap_width(data->bm),
					gfx_bitmap_height(data->bm),
					&mwidth,
					&mheight
				);


				if ((scaled = gfx_bitmap_create(mwidth, mheight, 32, BITMAPTAG_Clear, TRUE, BITMAPTAG_Friend, data->bm, TAG_DONE)))
				{
					gfx_scale(data->bm, scaled, mwidth, mheight,
						SCALETAG_Scale2x, TRUE,
						SCALETAG_Bilinear, TRUE,
						SCALETAG_Average, TRUE,
						SCALETAG_AspectX, NULL,
						SCALETAG_AspectY, NULL,
					TAG_DONE);
				}

				if ((selscaled = gfx_bitmap_create(mwidth, mheight, 32, BITMAPTAG_Clear, TRUE, BITMAPTAG_Friend, data->selbm, TAG_DONE)))
				{
					gfx_scale(data->selbm, selscaled, mwidth, mheight,
						SCALETAG_Scale2x, TRUE,
						SCALETAG_Bilinear, TRUE,
						SCALETAG_Average, TRUE,
						SCALETAG_AspectX, NULL,
						SCALETAG_AspectY, NULL,
					TAG_DONE);
				}

			}
			data->oldwidth = mwidth;
			data->oldheight = mheight;


		}
	
		if (scaled)
		{
			tbm = data->pressed?selscaled:scaled;
			txs = mwidth;
			tys = mheight;
		}
		else
		{
			tbm = data->pressed?data->selbm:data->bm;
			txs = gfx_bitmap_width(data->bm);
			tys = gfx_bitmap_height(data->bm);
		}


		gfx_blit(tbm, _rp(obj),
			BLITTAG_DstType, BLITVAL_DstType_RastPort,
			BLITTAG_DstX, mleft + (_mwidth( obj ) - txs) / 2,
			BLITTAG_DstY, mtop + (_mheight( obj ) - tys) / 2,
			BLITTAG_Alpha, 0xffffffff,
		TAG_DONE);

		data->scaled = scaled;
		data->selscaled = selscaled;
	}

	return (rc);
}

DEFMMETHOD(ContextMenuAdd)
{
	ULONG flags = 0;
	GETDATA;

	if (!getv(_parent(obj), MA_Panelgroup_Locked))
	{
		flags |= AS_PANEL_UNLOCKED;
	}

	data->cmenu	= contextmenu_build_simple(CM_PANELPOPUP, flags);

	if (data->cmenu != NULL)
	{
		if (data->menutitle != NULL)
		{
			set(data->cmenu, MUIA_Menu_Title, data->menutitle);
		}

		DoMethod(msg->menustrip, OM_ADDMEMBER, data->cmenu);
	}

	return (DOSUPER);
}


DEFMMETHOD(ContextMenuChoice)
{
	GETDATA;
	struct command_menu *cm;
	ULONG rc = 0;

	if (data->cmenu && DoMethod(data->cmenu, MUIM_FindObject, msg->item))
	{
		if ( ((cm = (struct command_menu *)getv(msg->item, MA_Menuitem_Command))) )
		{
			if (cm->name && *cm->name)
				execute_command(_parent(obj), cm->type, cm->args, NULL);
		}
	}
	else
		rc = DOSUPER;

	data->cmenu = NULL;

	return rc;
}


DEFSMETHOD(Panel_SaveConfig)
{
	APTR pl, pi; /* list, item */
	GETDATA;
	ASSERT(msg->pctx);

	if (!(pl = prefspool_item_get(msg->pctx, NULL, DSI_LISTPOOL_PANEL, NULL, NULL)))
	{
		pl = prefspool_item_add(msg->pctx, NULL, DSI_LISTPOOL_PANEL, NULL, NULL);
	}

	if (pl != NULL)
	{
		if (!(pi = prefspool_item_get(msg->pctx, pl, msg->index | DSF_LISTPOOL, NULL, NULL)))
		{
			pi = prefspool_item_add(msg->pctx, pl, msg->index | DSF_LISTPOOL, NULL, NULL);
		}
		if (pi != NULL)
		{
			setprefslong_lp(msg->pctx, pi, DSI_LISTPOOL_PANEL_DISTANCE, data->distance);
		}
	}
	
	return 0;
}

BEGINMTABLE
DECNEW
DECDISP
DECGET
DECSET
DECMMETHOD(AskMinMax)
DECMMETHOD(Draw)
DECMMETHOD(ContextMenuChoice)
DECMMETHOD(ContextMenuAdd)
DECSMETHOD(Panel_SaveConfig)
ENDMTABLE

DECSUBCLASS_NC(MUIC_Area, panelpopupclass)

